#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Apr  6 22:17:00 2019

@author: martindrech
"""
import numpy as np
from PyQt5 import QtCore
import time
import threading
from queue import Queue

class Helper_messager(QtCore.QObject):
    imageAquiredSignal = QtCore.pyqtSignal(int)


class AndorZyla:
    def __init__(self, num):
        self.ExposureTime = AndorFeats()
        self.FrameRate = AndorFeats()
        self.CameraAcquiring = AndorFeats()
        self.Binning = AndorFeats()
        self.ImageArea = AndorFeats()
        self.CameraName = 'Dummy'

        self.acq_queue = Queue(maxsize=10)
        self.helper = Helper_messager()
        self.current_image = None

        self.CameraAcquiring.setValue(False)

        self.acq_index_i = 0

        self.ExposureTime.setValue(0.5)
        self.FrameRate.setValue(10)
        self.Binning.setValue('1x1')
        self.ImageArea.setValue(50)


    def Init(self):
        print("Dummy camera initialized")

    def live_acquisition_loop(self):
        t = threading.currentThread()
        while getattr(t, "do_run", True):
            self.current_image = self._twoD_gaussian()
            self.acq_queue.put(self.current_image)
            self.helper.imageAquiredSignal.emit(self.acq_index_i)
            self.acq_index_i = self.acq_index_i + 1
            time.sleep(self.ExposureTime.getValue())
        print("stopping acq loop thread")

    def live_acquisition_configure(self):
        self.acq_index_i = 0

    def AcquisitionStart(self):
        self.CameraAcquiring.setValue(True)

    def AcquisitionStop(self):
        self.CameraAcquiring.setValue(False)

    def _flush(self):
        pass

    def _twoD_gaussian(self, lim=10, sigma=1, x0=0, y0=0):
        start = time.time()
        N = self.ImageArea.getValue()
        x, y = np.meshgrid(np.linspace(-lim, lim, N), np.linspace(-lim, lim, N))
        d = np.sqrt((x - x0) ** 2 + (y - y0) ** 2)
        g = np.exp(-((d) ** 2 / (2.0 * sigma ** 2)))
        noise = np.random.rand(N, N)
        end = time.time()
        print(end-start)
        time.sleep(self.ExposureTime.getValue())
        return g + noise


class AndorFeats:
    def __init__(self):
        self.value = None

    def getValue(self):
        return self.value

    def setValue(self, val):
        self.value = val

    def _twoD_gaussian(self, lim=10, N=100, sigma=1, x0=0, y0=0):
        x, y = np.meshgrid(np.linspace(-lim, lim, N), np.linspace(-lim, lim, N))
        d = np.sqrt((x - x0) ** 2 + (y - y0) ** 2)
        g = np.exp(-((d) ** 2 / (2.0 * sigma ** 2)))
        noise = np.random.rand(N, N)
        return g + noise
