#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr  4 11:10:20 2019

@author: martindrech


"""

import numpy as np
from PyQt5 import QtCore

from drivers.andorzyla import AndorZyla
#from dummyAndor import AndorZyla
import threading


class CameraWorker(QtCore.QObject):
    imageReadySignal = QtCore.pyqtSignal(np.ndarray, int)

    def __init__(self):
        super().__init__()

        self.cam = AndorZyla(0)
        self.cam.Init()
        #self.cam.ExposureTime.setValue(0.1)
        print("camera worker initialized")

    def make_connections(self, frontend):
        # frontend connections
        params = frontend.param_tree.p
        frontend.cameraButton.clicked.connect(lambda: self.run(params))        
        # internal connections
        self.cam.helper.imageAquiredSignal.connect(self.new_image_acquired)

    def __del__(self):
        print("adios camera worker")

    #@QtCore.pyqtSlot()
    def run(self, params):
        if self.cam.CameraAcquiring.getValue():
            self._stop_acquisition_loop()
            print("acquisition now stopping")

        else:
            self._configure_acq_parameters(params)
            self._start_acquisition_loop()
            self.acq_thread = threading.Thread(target=self.cam.live_acquisition_loop)
            self.acq_thread.start()
            print("acquisiting now starting")

    def _start_acquisition_loop(self):
        self.cam.live_acquisition_configure()
        self.cam.AcquisitionStart()
        print("frame rate: ", self.cam.FrameRate.getValue())
        print("Exp time: ", self.cam.ExposureTime.getValue())

    def _stop_acquisition_loop(self):
        self.acq_thread.do_run = False
        self.acq_thread.join()
        self.cam.AcquisitionStop()
        self.cam._flush()


    def _configure_acq_parameters(self, params):
        print('Acq parameters: ')
        for p in params.children()[0].children():
            print(p.name(), p.value())
        params.child('Basic acq parameters').child('Camera name').setValue(self.cam.CameraModel.getValue())
        self.cam.ExposureTime.setValue(params.child('Basic acq parameters')
                                   .child('Exposure Time').value()
                                  )
        self.cam.FrameRate.setValue(self.cam.FrameRate.max()
                                  )
        params.child('Basic acq parameters').child('Frame Rate').setValue(self.cam.FrameRate.getValue())
        self.cam.AOIBinning.setString(params.child('Basic acq parameters').child('Pixel Binning').value()
                                   )




    @QtCore.pyqtSlot(int)
    def new_image_acquired(self, acq_index):
        self.imageReadySignal.emit(self.cam.acq_queue.get(), acq_index)
